import logging
# Suppress logs below WARNING level
logging.basicConfig(level=logging.WARNING)
import struct
from spl.token.instructions import create_associated_token_account, get_associated_token_address, close_account, CloseAccountParams
from typing import Optional,Union
from solana.transaction import AccountMeta, Transaction
from spl.token.instructions import create_associated_token_account, get_associated_token_address
from solders.pubkey import Pubkey
from solders.instruction import Instruction
from solders.compute_budget import set_compute_unit_limit, set_compute_unit_price
from account_utils import payer_keypair,get_payer_keypair
from constants import *
from isolated_rpcs import sendTransaction, Hash,get_token_balance
from abstract_solcatcher import abstract_solana_rate_limited_call
from solana.rpc.types import TxOpts
from coin_data import get_pump_fun_data
from solana.rpc.types import TokenAccountOpts

mint = "AX4WqXbNvj8Um8SU1yGiVLFW1P3fwxEQR5gz2uv9pump"
TOKEN_PROGRAM = get_pubkey("TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA")
ASSOC_TOKEN_ACC_PROG = get_pubkey("ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL")
RENT = Pubkey.from_string("SysvarRent111111111111111111111111111111111")
def get_block_hash():
    recent_blockhash = abstract_solana_rate_limited_call('get_latest_blockhash').get('value', {}).get('blockhash')
    return Hash.from_string(recent_blockhash)

def buy(mint_str: str, sol_in: float = 0.001, slippage: int = 25) -> bool:
   
        # Get coin data
        coin_data = get_pump_fun_data(mint_str)
        if not coin_data:
            print("Failed to retrieve coin data...")
            return False

        owner = payer_keypair.pubkey()
        mint = Pubkey.from_string(mint_str)
        
        # Attempt to retrieve token account; if not, prepare to create it
        token_account = get_associated_token_address(owner, mint)
        account_data = abstract_solana_rate_limited_call('get_token_accounts_by_owner', owner, TokenAccountOpts(mint))
        token_account_instructions = None

        if not account_data.get('value'):
            # If token account does not exist, create the associated token account
            print("Creating new associated token account...")
            token_account_instructions = create_associated_token_account(owner, owner, mint)

        # Calculate amount of tokens to buy
        virtual_sol_reserves = coin_data['virtual_sol_reserves']
        virtual_token_reserves = coin_data['virtual_token_reserves']
        sol_in_lamports = sol_in * LAMPORTS_PER_SOL
        amount = int(sol_in_lamports * virtual_token_reserves / virtual_sol_reserves)

        # Calculate max SOL cost with slippage
        max_sol_cost = int(sol_in * (1 + slippage / 100) * LAMPORTS_PER_SOL)
        print("Max SOL Cost:", max_sol_cost / LAMPORTS_PER_SOL)

        # Define account keys for the transaction
        keys = [
            AccountMeta(pubkey=GLOBAL, is_signer=False, is_writable=False),
            AccountMeta(pubkey=FEE_RECIPIENT, is_signer=False, is_writable=True),
            AccountMeta(pubkey=Pubkey.from_string(coin_data['mint']), is_signer=False, is_writable=False),
            AccountMeta(pubkey=Pubkey.from_string(coin_data['bonding_curve']), is_signer=False, is_writable=True),
            AccountMeta(pubkey=Pubkey.from_string(coin_data['associated_bonding_curve']), is_signer=False, is_writable=True),
            AccountMeta(pubkey=token_account, is_signer=False, is_writable=True),
            AccountMeta(pubkey=owner, is_signer=True, is_writable=True),
            AccountMeta(pubkey=SYSTEM_PROGRAM, is_signer=False, is_writable=False),
            AccountMeta(pubkey=TOKEN_PROGRAM, is_signer=False, is_writable=False),
            AccountMeta(pubkey=RENT, is_signer=False, is_writable=False),
            AccountMeta(pubkey=EVENT_AUTHORITY, is_signer=False, is_writable=False),
            AccountMeta(pubkey=PUMP_FUN_PROGRAM, is_signer=False, is_writable=False)
        ]

        # Construct the swap instruction
        data = bytearray()
        data.extend(bytes.fromhex("66063d1201daebea"))  # Replace with actual instruction identifier
        data.extend(struct.pack('<Q', amount))
        data.extend(struct.pack('<Q', max_sol_cost))
        swap_instruction = Instruction(PUMP_FUN_PROGRAM, bytes(data), keys)

        # Construct and sign transaction
        txn = Transaction(recent_blockhash=get_block_hash(), fee_payer=owner)
        txn.add(set_compute_unit_price(UNIT_PRICE))
        txn.add(set_compute_unit_limit(UNIT_BUDGET))
        if token_account_instructions:
            txn.add(token_account_instructions)
        txn.add(swap_instruction)

        # Sign and send transaction
        txn.sign(payer_keypair)
        txn_sig = sendTransaction(txn, payer_keypair, opts=TxOpts(skip_preflight=True))
        print("Transaction Signature:", txn_sig)


        return txn_sig
      
def sell(
    mint_str: str,
    token_balance: Optional[Union[int, float]] = None,
    slippage: int = 25,
    close_token_account: bool = True,

) -> bool:
  
        # Get coin data
        coin_data = get_pump_fun_data(mint_str)
        if not coin_data:
            print("Failed to retrieve coin data...")
            return False

        payer_keypair = get_payer_keypair()
        owner = payer_keypair.pubkey()
        owner = payer_keypair.pubkey()
        mint = get_pubkey(coin_data['mint'])

        # Check if token account exists; create if necessary
        token_account = get_associated_token_address(owner, mint)
        account_data = abstract_solana_rate_limited_call('get_token_accounts_by_owner',  owner, TokenAccountOpts(mint))
        token_account_instructions = None

        if not account_data.get('value'):
            # If token account does not exist, create the associated token account
            print("Creating new associated token account...")
            create_account_instr = create_associated_token_account(owner, owner, mint)
            token_account_instructions = [create_account_instr]

        # Calculate token price
        sol_decimal = 10**9
        token_decimal = 10**6
        virtual_sol_reserves = coin_data['virtual_sol_reserves'] / sol_decimal
        virtual_token_reserves = coin_data['virtual_token_reserves'] / token_decimal
        token_price = virtual_sol_reserves / virtual_token_reserves
        print(f"Token Price: {token_price:.20f} SOL")

        # Get token balance if not provided
        if token_balance is None:
            token_balance = get_token_balance(mint_str,str(owner))
        if token_balance == 0:
            print("No tokens to sell.")
            return False

        seeds = [b"event_authority"]
        PUMP_FUN_PROGRAM_ID = Pubkey.from_string("6EF8rrecthR5Dkzon8Nwu78hRvfCKubJ14M5uBEwF6P")
        event_authority_pubkey, bump = Pubkey.find_program_address(seeds, PUMP_FUN_PROGRAM_ID)

        # Calculate amount and minimum SOL output
        amount = int(token_balance * token_decimal)
        sol_out = float(token_balance) * token_price
        slippage_adjustment = 1 - (slippage / 100)
        min_sol_output = int(sol_out * slippage_adjustment * LAMPORTS_PER_SOL)
        # Build account key list with corrected order
        keys = [
            AccountMeta(pubkey=GLOBAL, is_signer=False, is_writable=False),
            AccountMeta(pubkey=FEE_RECIPIENT, is_signer=False, is_writable=True),
            AccountMeta(pubkey=Pubkey.from_string(coin_data['mint']), is_signer=False, is_writable=False),
            AccountMeta(pubkey=Pubkey.from_string(coin_data['bonding_curve']), is_signer=False, is_writable=True),
            AccountMeta(pubkey=Pubkey.from_string(coin_data['associated_bonding_curve']), is_signer=False, is_writable=True),
            AccountMeta(pubkey=token_account, is_signer=False, is_writable=True),
            AccountMeta(pubkey=owner, is_signer=True, is_writable=True),
            AccountMeta(pubkey=get_pubkey(SYSTEM_PROGRAM), is_signer=False, is_writable=False), 
            AccountMeta(pubkey=ASSOC_TOKEN_ACC_PROG, is_signer=False, is_writable=False),
            AccountMeta(pubkey=TOKEN_PROGRAM, is_signer=False, is_writable=False),
            AccountMeta(pubkey=EVENT_AUTHORITY, is_signer=False, is_writable=False),
            AccountMeta(pubkey=get_pubkey(PUMP_FUN_PROGRAM), is_signer=False, is_writable=False)
        ]
        # Construct the swap instruction
        data = bytearray()
        data.extend(bytes.fromhex("33e685a4017f83ad"))  # Replace with actual instruction identifier for 'sell'
        data.extend(struct.pack('<Q', amount))
        data.extend(struct.pack('<Q', min_sol_output))
        swap_instruction = Instruction(get_pubkey(PUMP_FUN_PROGRAM), bytes(data), keys)

        # Construct and sign transaction
        recent_blockhash = get_block_hash()
        txn = Transaction(recent_blockhash=recent_blockhash, fee_payer=owner)
        txn.add(set_compute_unit_price(UNIT_PRICE))
        txn.add(set_compute_unit_limit(UNIT_BUDGET))

        if token_account_instructions:
            txn.extend(token_account_instructions)

        txn.add(swap_instruction)

        # Conditionally add the close account instruction
        if close_token_account:
            close_instr = close_account(CloseAccountParams(
                program_id=get_pubkey(TOKEN_PROGRAM),
                account=token_account,
                dest=owner,
                owner=owner
            ))
            txn.add(close_instr)

        txn.sign(payer_keypair)

        # Send and confirm transaction
        txn_sig = sendTransaction(txn, payer_keypair, opts=TxOpts(skip_preflight=True))
        print("Transaction Signature:", txn_sig)

        return txn_sig



   
sell(mint)



import struct
from solana.transaction import AccountMeta, Transaction
from spl.token.instructions import create_associated_token_account, get_associated_token_address, close_account, CloseAccountParams
from solders.instruction import Instruction #type: ignore
from solders.compute_budget import set_compute_unit_limit, set_compute_unit_price #type: ignore
from account_utils import get_payer_keypair
from constants import *
from isolated_rpcs import sendTransaction,get_token_balance,get_block_hash,confirm_txn
from abstract_solcatcher import *
from solana.rpc.types import TokenAccountOpts,TxOpts
from coin_data import get_pump_fun_data
from typing import Optional, Union
from abstract_solana import *
from abstract_database import *
from abstract_solana import Client,get_insert_list
from abstract_apis import postRequest,getRequest,postRpcRequest,get_async_response,postRequest

def start_txn(data,keys,owner,token_account_instructions):
    swap_instruction = Instruction(get_pubkey(PUMP_FUN_PROGRAM), bytes(data), keys)
    # Construct and sign transaction
    recent_blockhash = get_block_hash()
    txn = Transaction(recent_blockhash=recent_blockhash, fee_payer=owner)
    txn.add(set_compute_unit_price(UNIT_PRICE))
    txn.add(set_compute_unit_limit(UNIT_BUDGET))
    if token_account_instructions:
        txn.add(token_account_instructions)
    txn.add(swap_instruction)
    return txn
def get_coin_data(mint_str,coin_data,payer_keypair=None,fileName=None,envKey=None):
    
    payer_keypair = payer_keypair or get_payer_keypair(fileName=fileName,envKey=envKey)
    owner = payer_keypair.pubkey()
    mint = get_pubkey(mint_str)
    # Attempt to retrieve token account; if not, prepare to create it
    token_account = get_associated_token_address(owner, mint)
    
    account_data = call_solcatcher_db_api('get_token_accounts_by_owner', owner, TokenAccountOpts(mint))
    token_account_instructions = None
   
    if not account_data.get('value'):
        # If token account does not exist, create the associated token account
        print("Creating new associated token account...")
        token_account_instructions = create_associated_token_account(owner, owner, mint)
    return owner, owner, mint,token_account,token_account_instructions
def send_txn(txn,payer_keypair=None,fileName=None,envKey=None):
    payer_keypair = payer_keypair or get_payer_keypair(fileName=fileName,envKey=envKey)
    txn.sign(payer_keypair)
    # Send and confirm transaction
    txn_sig = sendTransaction(txn, payer_keypair, opts=TxOpts(skip_preflight=True))
    print("Transaction Signature", txn_sig)
    return txn_sig
def buy(mint_str: str, sol_in: float = 0.001, slippage: int = 25,payer_keypair=None,fileName=None,envKey=None) -> bool:
    try:
        # Get coin data
        coin_data = get_pump_fun_data(mint_str)
        if not coin_data:
            print("Failed to retrieve coin data...")
            return False
        payer_keypair = payer_keypair or get_payer_keypair()
        owner = payer_keypair.pubkey()
        mint = Pubkey.from_string(mint_str)
        
        # Attempt to retrieve token account; if not, prepare to create it
        token_account = get_associated_token_address(owner, mint)
        account_data = rate_limit_solcatcher_api('get_token_accounts_by_owner', owner, TokenAccountOpts(mint))
        token_account_instructions = None

        if not account_data.get('value'):
            # If token account does not exist, create the associated token account
            print("Creating new associated token account...")
            token_account_instructions = create_associated_token_account(owner, owner, mint)

        # Calculate amount of tokens to buy
        virtual_sol_reserves = coin_data['virtual_sol_reserves']
        virtual_token_reserves = coin_data['virtual_token_reserves']
        sol_in_lamports = sol_in * LAMPORTS_PER_SOL
        amount = int(sol_in_lamports * virtual_token_reserves / virtual_sol_reserves)

        # Calculate max SOL cost with slippage
        max_sol_cost = int(sol_in * (1 + slippage / 100) * LAMPORTS_PER_SOL)
        print("Max SOL Cost:", max_sol_cost / LAMPORTS_PER_SOL)

        # Define account keys for the transaction
        keys = [
            AccountMeta(pubkey=get_pubkey(GLOBAL), is_signer=False, is_writable=False),
            AccountMeta(pubkey=get_pubkey(FEE_RECIPIENT), is_signer=False, is_writable=True),
            AccountMeta(pubkey=get_pubkey(coin_data['mint']), is_signer=False, is_writable=False),
            AccountMeta(pubkey=get_pubkey(coin_data['bonding_curve']), is_signer=False, is_writable=True),
            AccountMeta(pubkey=get_pubkey(coin_data['associated_bonding_curve']), is_signer=False, is_writable=True),
            AccountMeta(pubkey=get_pubkey(token_account), is_signer=False, is_writable=True),
            AccountMeta(pubkey=get_pubkey(owner), is_signer=True, is_writable=True),
            AccountMeta(pubkey=get_pubkey(SYSTEM_PROGRAM), is_signer=False, is_writable=False), 
            AccountMeta(pubkey=get_pubkey(ASSOC_TOKEN_ACC_PROG), is_signer=False, is_writable=False),
            AccountMeta(pubkey=get_pubkey(TOKEN_PROGRAM), is_signer=False, is_writable=False),
            AccountMeta(pubkey=get_pubkey(EVENT_AUTHORITY), is_signer=False, is_writable=False),
            AccountMeta(pubkey=get_pubkey(PUMP_FUN_PROGRAM), is_signer=False, is_writable=False)
        ]

        # Construct the swap instruction
        data = bytearray()
        data.extend(bytes.fromhex("66063d1201daebea"))  # Replace with actual instruction identifier
        data.extend(struct.pack('<Q', amount))
        data.extend(struct.pack('<Q', max_sol_cost))
        swap_instruction = Instruction(get_pubkey(PUMP_FUN_PROGRAM), bytes(data), keys)

        # Construct and sign transaction
        txn = Transaction(recent_blockhash=get_block_hash(), fee_payer=owner)
        txn.add(set_compute_unit_price(UNIT_PRICE))
        txn.add(set_compute_unit_limit(UNIT_BUDGET))
        if token_account_instructions:
            txn.add(token_account_instructions)
        txn.add(swap_instruction)

        # Sign and send transaction
        txn.sign(payer_keypair)
        txn_sig = sendTransaction(txn, payer_keypair, opts=TxOpts(skip_preflight=True))
        print("Transaction Signature:", txn_sig)

        return txn_sig
        
    except Exception as e:
        print("Transaction failed:", e)
        return False


def sell(mint_str: str, token_balance: Optional[Union[int, float]] = None, slippage: int = 25, close_token_account: bool = True,payer_keypair=None,fileName=None,envKey=None) -> bool:
    try:
        # Get coin data
        coin_data = get_pump_fun_data(mint_str)
        if not coin_data:
            print("Failed to retrieve coin data...")
            return False

        payer_keypair = payer_keypair or get_payer_keypair()
        owner = payer_keypair.pubkey()
        mint = Pubkey.from_string(mint_str)

        # Check if token account exists; create if necessary
        token_account = get_associated_token_address(owner, mint)
        try:
            account_data = abstract_solana_rate_limited_call('solcatcher', 'get_token_accounts_by_owner', owner, TokenAccountOpts(mint))
            if not account_data.get('value'):  # If no token account exists
                create_account_instr = create_associated_token_account(owner, owner, mint)
                token_account_instructions = [create_account_instr]
        except Exception as e:
            print("Token account check or creation failed:", e)
            return False

        # Calculate token price
        sol_decimal = 10**9
        token_decimal = 10**6
        virtual_sol_reserves = coin_data['virtual_sol_reserves'] / sol_decimal
        virtual_token_reserves = coin_data['virtual_token_reserves'] / token_decimal
        token_price = virtual_sol_reserves / virtual_token_reserves
        print(f"Token Price: {token_price:.20f} SOL")

        # Get token balance if not provided
        if token_balance is None:
            token_balance = get_token_balance(mint_str)
        if token_balance == 0:
            print("No tokens to sell.")
            return False

        # Calculate amount and minimum SOL output
        amount = int(token_balance * token_decimal)
        sol_out = float(token_balance) * token_price
        slippage_adjustment = 1 - (slippage / 100)
        min_sol_output = int(sol_out * slippage_adjustment * LAMPORTS_PER_SOL)

        # Build account key list
        keys = [
            AccountMeta(pubkey=GLOBAL, is_signer=False, is_writable=False),
            AccountMeta(pubkey=FEE_RECIPIENT, is_signer=False, is_writable=True),
            AccountMeta(pubkey=Pubkey.from_string(coin_data['mint']), is_signer=False, is_writable=False),
            AccountMeta(pubkey=Pubkey.from_string(coin_data['bonding_curve']), is_signer=False, is_writable=True),
            AccountMeta(pubkey=Pubkey.from_string(coin_data['associated_bonding_curve']), is_signer=False, is_writable=True),
            AccountMeta(pubkey=token_account, is_signer=False, is_writable=True),
            AccountMeta(pubkey=owner, is_signer=True, is_writable=True),
            AccountMeta(pubkey=SYSTEM_PROGRAM, is_signer=False, is_writable=False), 
            AccountMeta(pubkey=ASSOC_TOKEN_ACC_PROG, is_signer=False, is_writable=False),
            AccountMeta(pubkey=TOKEN_PROGRAM, is_signer=False, is_writable=False),
            AccountMeta(pubkey=EVENT_AUTHORITY, is_signer=False, is_writable=False),
            AccountMeta(pubkey=PUMP_FUN_PROGRAM, is_signer=False, is_writable=False)
        ]

        # Construct the swap instruction
        data = bytearray()
        data.extend(bytes.fromhex("33e685a4017f83ad"))
        data.extend(struct.pack('<Q', amount))
        data.extend(struct.pack('<Q', min_sol_output))
        swap_instruction = Instruction(PUMP_FUN_PROGRAM, bytes(data), keys)

        # Construct and sign transaction
        recent_blockhash = get_block_hash()
        txn = Transaction(recent_blockhash=recent_blockhash, fee_payer=owner)
        txn.add(set_compute_unit_price(UNIT_PRICE))
        txn.add(set_compute_unit_limit(UNIT_BUDGET))
        
        if 'token_account_instructions' in locals():
            txn.extend(token_account_instructions)
        
        txn.add(swap_instruction)

        # Conditionally add the close account instruction
        if close_token_account:
            close_instr = close_account(CloseAccountParams(TOKEN_PROGRAM, token_account, owner, owner))
            txn.add(close_instr)

        txn.sign(payer_keypair)

        # Send and confirm transaction
        txn_sig = sendTransaction(txn, payer_keypair, opts=TxOpts(skip_preflight=True))
        print("Transaction Signature", txn_sig)
        confirm = confirm_txn(txn_sig)
        print(confirm)
        return True

    except Exception as e:
        print(f"Sell function encountered an error: {e}")
        return False


    



from .account_utils import *
from .coin_data import *
from .constants import *
from .flaSKIT import *
from .instruction_utils import *
from .isolated_rpcs import *
from .pump_fun import *
from .swapit import *
from .swap_utils import *
from .txn_utils import *
from .utils import *



from flask import Blueprint, request, jsonify, send_file
from abstract_utilities import *
from abstract_pandas import *
import os
def getAbsPath():
  return os.path.dirname(os.path.abspath(__name__))
def makeAbsFile(path):
  return os.path.join(getAbsPath(),path)
def get_solanaTrackDir():
  trackerDir = makeAbsFile('solanaTrackDir')
  os.makedirs(trackerDir,exist_ok=True)
  return trackerDir
def get_solanaDfPath():
  return os.path.join(get_solanaTrackDir(),'solanaTracker.xlsx')
def getSolanaDfData():
  solanaDfPath = get_solanaDfPath()
  if not os.path.isfile(solanaDfPath):
    safe_excel_save(get_df([{}]),solanaDfPath)
  return get_df(solanaDfPath)
  
input(getSolanaDfData())
solana_tracker = Blueprint('solana_tracker', __name__)

# Directory to save the master Excel record
EXCEL_DIR = '/path/to/your/excel/files'
EXCEL_FILE = os.path.join(EXCEL_DIR, 'solana_swaps_master.xlsx')

@solana_tracker.route('/track_swap', methods=['POST'])
def track_swap():
    data = request.json
    
    # Assuming `data` contains relevant swap details
    swap_df = pd.DataFrame([data])
    
    # Check if the master Excel file exists
    if os.path.exists(EXCEL_FILE):
        master_df = pd.read_excel(EXCEL_FILE)
        master_df = pd.concat([master_df, swap_df], ignore_index=True)
    else:
        master_df = swap_df
    
    # Save the updated dataframe to Excel
    master_df.to_excel(EXCEL_FILE, index=False)
    
    return jsonify({"message": "Swap tracked successfully"}), 201

@solana_tracker.route('/get_swaps', methods=['GET'])
def get_swaps():
    if os.path.exists(EXCEL_FILE):
        return send_file(EXCEL_FILE, as_attachment=True)
    else:
        return jsonify({"message": "No swaps tracked yet."}), 404



from solders.pubkey import Pubkey
from solana.transaction import Signature
from abstract_utilities import safe_json_loads
def isListZero(list_obj):
    if isinstance(list_obj,list) and len(list_obj)>0:
        return list_obj[0]
    return list_obj
def get_pubkey(address):
    # Convert address string to Pubkey, with error handling
    if isinstance(address, Pubkey):
        return address
    address = str(address)
    try:
        return Pubkey.from_string(address)
    except Exception as e:
        print(f"Invalid address format: {e}")
        return None
def get_pubKey(address):
    # Convert address string to Pubkey, with error handling
    if isinstance(address, Pubkey):
        return address
    address = str(address)
    try:
        return Pubkey.from_string(address)
    except Exception as e:
        print(f"Invalid address format: {e}")
        return None
def get_sigKey(txn_sig):
    if isinstance(txn_sig, Signature):
        return txn_sig
    txn_sig = str(txn_sig)
    try:
        return Signature.from_string(txn_sig)
    except Exception as e:
        print(f"Invalid address format: {e}")
        return None



import requests,time, json, struct, os, base58, sys, base64
from .utils import isListZero,Pubkey,get_sigKey
from .account_utils import check_existing_token_account
from solana.rpc.types import TokenAccountOpts, TxOpts
from solana.transaction import Transaction
from abstract_utilities import get_any_value,safe_json_loads
from abstract_apis import postRpcRequest
from abstract_solcatcher import *
from abstract_solcatcher.database_calls.checkManager import rate_limit
from solders.hash import Hash
def call_solcatcher_api(method,*args,**kwargs):
  return CheckSolcatcher().call_solcatcher_api(method,*args,**kwargs)
def rate_limit(method=None,status_code=False):
  return CheckSolcatcher().get_rate_url(method=method,status_code=status_code)
def log_response(method=None,response=None,status_code=False):
  return CheckSolcatcher().log_response(method=method,response=response,status_code=status_code)
def rate_limit_solcatcher_api(method,*args,**kwargs):
  response = call_solcatcher_api(method,*args,**kwargs)
  logged_response = log_response(method=method,response=response,status_code=kwargs.get('status_code'))
  return response
def exec_request(method,params):
    url = CheckSolcatcher().rateLimiter.get_url(method)
    payload = {
        "jsonrpc": "2.0",
        "id": 1,
        "method": method,
        "params": params
    }
    
    response = postRpcRequest(
        url=url,
        **payload
    )
    rate_limit(method,response)
    return response
def get_block_hash():
    recent_blockhash = abstract_solana_rate_limited_call('get_latest_blockhash').get('value', {}).get('blockhash')
    return Hash.from_string(recent_blockhash)
def get_token_accounts_by_owner(owner,mint):
    return abstract_solana_rate_limited_call('get_token_accounts_by_owner', owner, TokenAccountOpts(mint))
def get_value(response_json):
    try:
        account_info = response_json.get("value", {})
        return account_info
    except Exception as e:
        print(f"Error in retrieving account info: {e}")
        return None
def getAccountInfo(mint_str: str) -> dict:
    # Prepare the RPC request payload
    
    method,params = "getAccountInfo",[mint_str, {"encoding": "base64"}]  # Use base64 for direct decoding
    # Send the transaction
    
    response_json = exec_request(method,params)
    # Parse the JSON response
    return get_value(response_json)

def get_account_by_owner(payer, mint_str: str) -> dict:
    method,params = "getTokenAccountsByOwner",[payer, {"mint": mint_str}, {"encoding": "jsonParsed"}]
    response_json = exec_request(method,params)
    accounts = response_json.get("value", {})
    if accounts:
        return accounts[0]  # Return the first account found
    return None

def get_block_hash():
    recent_blockhash = call_solcatcher_api('get_latest_blockhash').get('value',{}).get('blockhash')
    recent_blockhash = Hash.from_string(recent_blockhash)
    return recent_blockhash

def get_account_by_owner(payer,mint_str: str) -> dict:
    # Prepare the RPC request payload
    method,params = "getTokenAccountsByOwner",[payer, {"mint":mint_str},{"encoding": "jsonParsed"}]
    # Send the transaction
    response_json = exec_request(method,params)
    response_json = isListZero(response_json).get('value', isListZero(response_json))
    response_json = isListZero(response_json).get('data', isListZero(response_json))
    # Return the result or the entire response in case of error
    return response_json
def get_token_balance(mint_str: str,pubkey_str):
    method,params = "getTokenAccountsByOwner",[pubkey_str,{"mint": mint_str},{"encoding": "jsonParsed"}]
    response_json = exec_request(method,params)
    ui_amount = get_any_value(response_json, "uiAmount")
    return float(ui_amount or 0)
def sendTransaction(txn: Transaction, payer_keypair,commitment=None, opts=TxOpts(skip_preflight=True)) -> dict:
    # Sign the transaction
    txn.sign(payer_keypair)
    # Serialize the transaction to a base64 string
    txn_base64 = base58.b58encode(txn.serialize()).decode('utf-8')
    # Prepare the RPC request payload
    method,params = "sendTransaction",[txn_base64, {"skipPreflight": opts.skip_preflight, "preflightCommitment":commitment or  "finalized"}]
    # Return the result or the entire response in case of error
    return exec_request(method,params)
def get_transaction(signature,commitment,maxSupportedTransactionVersion=0,max_supported_transaction_version=0):
    # Prepare the RPC request payload
    method,params = "getTransaction",[signature, {"commitment":commitment or "finalized","maxSupportedTransactionVersion":maxSupportedTransactionVersion or  max_supported_transaction_version}]
    # Return the result or the entire response in case of error
    return exec_request(method,params)
def confirm_txn(txn_sig, max_retries=20, retry_interval=3):
    retries = 0
    txn_sig = get_sigKey(txn_sig)
    while retries < max_retries:
            print(txn_sig)
            txn_res = get_transaction(str(txn_sig), commitment="confirmed", max_supported_transaction_version=0)
            if txn_res:
                txn_res = safe_json_loads(txn_res)
                if isinstance(txn_res,dict):
                    txn_json = txn_res.get('meta')
                    if txn_json and isinstance(txn_json,dict):
                        if txn_json['err'] is None:
                            print("Transaction confirmed... try count:", retries+1)
                            return True
                        print("Error: Transaction not confirmed. Retrying...")
                        if txn_json['err']:
                            print("Transaction failed.")
                            return False

    print("Max retries reached. Transaction confirmation failed.")
    return None
